@@ -55,13 +55,13 @@ struct i_poll_events;
 //  This class implements socket polling mechanism using the Linux-specific
 //  epoll mechanism.
 
-class epoll_t : public worker_poller_base_t
+class epoll_t ZMQ_FINAL : public worker_poller_base_t
 {
   public:
     typedef void *handle_t;
 
     epoll_t (const thread_ctx_t &ctx_);
-    ~epoll_t ();
+    ~epoll_t () ZMQ_OVERRIDE;
 
     //  "poller" concept.
     handle_t add_fd (fd_t fd_, zmq::i_poll_events *events_);
@@ -103,8 +103,7 @@ class epoll_t : public worker_poller_base_t
     typedef std::vector<poll_entry_t *> retired_t;
     retired_t _retired;
 
-    epoll_t (const epoll_t &);
-    const epoll_t &operator= (const epoll_t &);
+    ZMQ_NON_COPYABLE_NOR_MOVABLE (epoll_t)
 };
 
 typedef epoll_t poller_t;
