@@ -39,7 +39,7 @@ class ctx_t;
 class pipe_t;
 class msg_t;
 
-class gather_t : public socket_base_t
+class gather_t ZMQ_FINAL : public socket_base_t
 {
   public:
     gather_t (zmq::ctx_t *parent_, uint32_t tid_, int sid_);
@@ -59,8 +59,7 @@ class gather_t : public socket_base_t
     //  Fair queueing object for inbound pipes.
     fq_t _fq;
 
-    gather_t (const gather_t &);
-    const gather_t &operator= (const gather_t &);
+    ZMQ_NON_COPYABLE_NOR_MOVABLE (gather_t)
 };
 }
 
