@@ -41,7 +41,7 @@ class msg_t;
 class pipe_t;
 class io_thread_t;
 
-class pair_t : public socket_base_t
+class pair_t ZMQ_FINAL : public socket_base_t
 {
   public:
     pair_t (zmq::ctx_t *parent_, uint32_t tid_, int sid_);
@@ -64,8 +64,7 @@ class pair_t : public socket_base_t
 
     zmq::pipe_t *_last_in;
 
-    pair_t (const pair_t &);
-    const pair_t &operator= (const pair_t &);
+    ZMQ_NON_COPYABLE_NOR_MOVABLE (pair_t)
 };
 }
 
