@@ -33,8 +33,12 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <secoid.h>
 #include <sechash.h>
 #define SHA_DIGEST_LENGTH 20
-#else
+#elif defined ZMQ_USE_BUILTIN_SHA1
 #include "../external/sha1/sha1.h"
+#elif defined ZMQ_USE_GNUTLS
+#define SHA_DIGEST_LENGTH 20
+#include <gnutls/gnutls.h>
+#include <gnutls/crypto.h>
 #endif
 
 #if !defined ZMQ_HAVE_WINDOWS
@@ -48,6 +52,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #endif
 #endif
 
+#include <cstring>
+
 #include "tcp.hpp"
 #include "ws_engine.hpp"
 #include "session_base.hpp"
@@ -56,9 +62,37 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "random.hpp"
 #include "ws_decoder.hpp"
 #include "ws_encoder.hpp"
+#include "null_mechanism.hpp"
+#include "plain_server.hpp"
+#include "plain_client.hpp"
+
+#ifdef ZMQ_HAVE_CURVE
+#include "curve_client.hpp"
+#include "curve_server.hpp"
+#endif
 
 #ifdef ZMQ_HAVE_WINDOWS
 #define strcasecmp _stricmp
+#define strtok_r strtok_s
+#else
+#ifdef ZMQ_HAVE_LIBBSD
+#include <bsd/string.h>
+#elif !defined(ZMQ_HAVE_STRLCPY)
+static size_t strlcpy (char *dest_, const char *src_, const size_t dest_size_)
+{
+    size_t remain = dest_size_;
+    for (; remain && *src_; --remain, ++src_, ++dest_) {
+        *dest_ = *src_;
+    }
+    return dest_size_ - remain;
+}
+#endif
+template <size_t size>
+static int strcpy_s (char (&dest_)[size], const char *const src_)
+{
+    const size_t res = strlcpy (dest_, src_, size);
+    return res >= size ? ERANGE : 0;
+}
 #endif
 
 //  OSX uses a different name for this socket option
@@ -71,15 +105,15 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #endif
 
 static int
-encode_base64 (const unsigned char *in, int in_len, char *out, int out_len);
+encode_base64 (const unsigned char *in_, int in_len_, char *out_, int out_len_);
 
-static void compute_accept_key (char *key,
-                                unsigned char output[SHA_DIGEST_LENGTH]);
+static void compute_accept_key (char *key_,
+                                unsigned char hash_[SHA_DIGEST_LENGTH]);
 
 zmq::ws_engine_t::ws_engine_t (fd_t fd_,
                                const options_t &options_,
                                const endpoint_uri_pair_t &endpoint_uri_pair_,
-                               ws_address_t &address_,
+                               const ws_address_t &address_,
                                bool client_) :
     stream_engine_base_t (fd_, options_, endpoint_uri_pair_),
     _client (client_),
@@ -89,27 +123,49 @@ zmq::ws_engine_t::ws_engine_t (fd_t fd_,
     _header_name_position (0),
     _header_value_position (0),
     _header_upgrade_websocket (false),
-    _header_connection_upgrade (false)
+    _header_connection_upgrade (false),
+    _heartbeat_timeout (0)
 {
     memset (_websocket_key, 0, MAX_HEADER_VALUE_LENGTH + 1);
     memset (_websocket_accept, 0, MAX_HEADER_VALUE_LENGTH + 1);
-    memset (_websocket_protocol, 0, MAX_HEADER_VALUE_LENGTH + 1);
+    memset (_websocket_protocol, 0, 256);
 
-    _next_msg = static_cast<int (stream_engine_base_t::*) (msg_t *)> (
-      &ws_engine_t::routing_id_msg);
-    _process_msg = static_cast<int (stream_engine_base_t::*) (msg_t *)> (
-      &ws_engine_t::process_routing_id_msg);
+    _next_msg = &ws_engine_t::next_handshake_command;
+    _process_msg = &ws_engine_t::process_handshake_command;
+    _close_msg.init ();
+
+    if (_options.heartbeat_interval > 0) {
+        _heartbeat_timeout = _options.heartbeat_timeout;
+        if (_heartbeat_timeout == -1)
+            _heartbeat_timeout = _options.heartbeat_interval;
+    }
 }
 
 zmq::ws_engine_t::~ws_engine_t ()
 {
+    _close_msg.close ();
 }
 
-void zmq::ws_engine_t::plug_internal ()
+void zmq::ws_engine_t::start_ws_handshake ()
 {
     if (_client) {
+        const char *protocol;
+        if (_options.mechanism == ZMQ_NULL)
+            protocol = "ZWS2.0/NULL,ZWS2.0";
+        else if (_options.mechanism == ZMQ_PLAIN)
+            protocol = "ZWS2.0/PLAIN";
+#ifdef ZMQ_HAVE_CURVE
+        else if (_options.mechanism == ZMQ_CURVE)
+            protocol = "ZWS2.0/CURVE";
+#endif
+        else {
+            // Avoid unitialized variable error breaking UWP build
+            protocol = "";
+            assert (false);
+        }
+
         unsigned char nonce[16];
-        int *p = (int *) nonce;
+        int *p = reinterpret_cast<int *> (nonce);
 
         // The nonce doesn't have to be secure one, it is just use to avoid proxy cache
         *p = zmq::generate_random ();
@@ -121,32 +177,38 @@ void zmq::ws_engine_t::plug_internal ()
           encode_base64 (nonce, 16, _websocket_key, MAX_HEADER_VALUE_LENGTH);
         assert (size > 0);
 
-        size = snprintf ((char *) _write_buffer, WS_BUFFER_SIZE,
-                         "GET %s HTTP/1.1\r\n"
-                         "Host: %s\r\n"
-                         "Upgrade: websocket\r\n"
-                         "Connection: Upgrade\r\n"
-                         "Sec-WebSocket-Key: %s\r\n"
-                         "Sec-WebSocket-Protocol: ZWS2.0\r\n"
-                         "Sec-WebSocket-Version: 13\r\n\r\n",
-                         _address.path (), _address.host (), _websocket_key);
+        size = snprintf (
+          reinterpret_cast<char *> (_write_buffer), WS_BUFFER_SIZE,
+          "GET %s HTTP/1.1\r\n"
+          "Host: %s\r\n"
+          "Upgrade: websocket\r\n"
+          "Connection: Upgrade\r\n"
+          "Sec-WebSocket-Key: %s\r\n"
+          "Sec-WebSocket-Protocol: %s\r\n"
+          "Sec-WebSocket-Version: 13\r\n\r\n",
+          _address.path (), _address.host (), _websocket_key, protocol);
         assert (size > 0 && size < WS_BUFFER_SIZE);
         _outpos = _write_buffer;
         _outsize = size;
         set_pollout ();
     }
+}
 
+void zmq::ws_engine_t::plug_internal ()
+{
+    start_ws_handshake ();
     set_pollin ();
     in_event ();
 }
 
 int zmq::ws_engine_t::routing_id_msg (msg_t *msg_)
 {
-    int rc = msg_->init_size (_options.routing_id_size);
+    const int rc = msg_->init_size (_options.routing_id_size);
     errno_assert (rc == 0);
     if (_options.routing_id_size > 0)
         memcpy (msg_->data (), _options.routing_id, _options.routing_id_size);
     _next_msg = &ws_engine_t::pull_msg_from_session;
+
     return 0;
 }
 
@@ -154,7 +216,7 @@ int zmq::ws_engine_t::process_routing_id_msg (msg_t *msg_)
 {
     if (_options.recv_routing_id) {
         msg_->set_flags (msg_t::routing_id);
-        int rc = session ()->push_msg (msg_);
+        const int rc = session ()->push_msg (msg_);
         errno_assert (rc == 0);
     } else {
         int rc = msg_->close ();
@@ -168,6 +230,56 @@ int zmq::ws_engine_t::process_routing_id_msg (msg_t *msg_)
     return 0;
 }
 
+bool zmq::ws_engine_t::select_protocol (const char *protocol_)
+{
+    if (_options.mechanism == ZMQ_NULL && (strcmp ("ZWS2.0", protocol_) == 0)) {
+        _next_msg = static_cast<int (stream_engine_base_t::*) (msg_t *)> (
+          &ws_engine_t::routing_id_msg);
+        _process_msg = static_cast<int (stream_engine_base_t::*) (msg_t *)> (
+          &ws_engine_t::process_routing_id_msg);
+
+        // No mechanism in place, enabling heartbeat
+        if (_options.heartbeat_interval > 0 && !_has_heartbeat_timer) {
+            add_timer (_options.heartbeat_interval, heartbeat_ivl_timer_id);
+            _has_heartbeat_timer = true;
+        }
+
+        return true;
+    }
+    if (_options.mechanism == ZMQ_NULL
+        && strcmp ("ZWS2.0/NULL", protocol_) == 0) {
+        _mechanism = new (std::nothrow)
+          null_mechanism_t (session (), _peer_address, _options);
+        alloc_assert (_mechanism);
+        return true;
+    } else if (_options.mechanism == ZMQ_PLAIN
+               && strcmp ("ZWS2.0/PLAIN", protocol_) == 0) {
+        if (_options.as_server)
+            _mechanism = new (std::nothrow)
+              plain_server_t (session (), _peer_address, _options);
+        else
+            _mechanism =
+              new (std::nothrow) plain_client_t (session (), _options);
+        alloc_assert (_mechanism);
+        return true;
+    }
+#ifdef ZMQ_HAVE_CURVE
+    else if (_options.mechanism == ZMQ_CURVE
+             && strcmp ("ZWS2.0/CURVE", protocol_) == 0) {
+        if (_options.as_server)
+            _mechanism = new (std::nothrow)
+              curve_server_t (session (), _peer_address, _options);
+        else
+            _mechanism =
+              new (std::nothrow) curve_client_t (session (), _options);
+        alloc_assert (_mechanism);
+        return true;
+    }
+#endif
+
+    return false;
+}
+
 bool zmq::ws_engine_t::handshake ()
 {
     bool complete;
@@ -197,12 +309,8 @@ bool zmq::ws_engine_t::handshake ()
 
 bool zmq::ws_engine_t::server_handshake ()
 {
-    int nbytes = tcp_read (_read_buffer, WS_BUFFER_SIZE);
-    if (nbytes == 0) {
-        errno = EPIPE;
-        error (zmq::i_engine::connection_error);
-        return false;
-    } else if (nbytes == -1) {
+    const int nbytes = read (_read_buffer, WS_BUFFER_SIZE);
+    if (nbytes == -1) {
         if (errno != EAGAIN)
             error (zmq::i_engine::connection_error);
         return false;
@@ -212,7 +320,7 @@ bool zmq::ws_engine_t::server_handshake ()
     _insize = nbytes;
 
     while (_insize > 0) {
-        char c = (char) *_inpos;
+        const char c = static_cast<char> (*_inpos);
 
         switch (_server_handshake_state) {
             case handshake_initial:
@@ -325,7 +433,7 @@ bool zmq::ws_engine_t::server_handshake ()
                         _server_handshake_state = handshake_error;
                         break;
                     default:
-                        _header_name[0] = (char) c;
+                        _header_name[0] = c;
                         _header_name_position = 1;
                         _server_handshake_state = header_field_name;
                         break;
@@ -373,24 +481,25 @@ bool zmq::ws_engine_t::server_handshake ()
                           strcasecmp ("upgrade", _header_value) == 0;
                     else if (strcasecmp ("Sec-WebSocket-Key", _header_name)
                              == 0)
-                        strcpy (_websocket_key, _header_value);
+                        strcpy_s (_websocket_key, _header_value);
                     else if (strcasecmp ("Sec-WebSocket-Protocol", _header_name)
                              == 0) {
                         // Currently only the ZWS2.0 is supported
                         // Sec-WebSocket-Protocol can appear multiple times or be a comma separated list
                         // if _websocket_protocol is already set we skip the check
                         if (_websocket_protocol[0] == '\0') {
-                            char *p = strtok (_header_value, ",");
+                            char *rest = 0;
+                            char *p = strtok_r (_header_value, ",", &rest);
                             while (p != NULL) {
                                 if (*p == ' ')
                                     p++;
 
-                                if (strcmp ("ZWS2.0", p) == 0) {
-                                    strcpy (_websocket_protocol, p);
+                                if (select_protocol (p)) {
+                                    strcpy_s (_websocket_protocol, p);
                                     break;
                                 }
 
-                                p = strtok (NULL, ",");
+                                p = strtok_r (NULL, ",", &rest);
                             }
                         }
                     }
@@ -420,14 +529,15 @@ bool zmq::ws_engine_t::server_handshake ()
                         unsigned char hash[SHA_DIGEST_LENGTH];
                         compute_accept_key (_websocket_key, hash);
 
-                        int accept_key_len = encode_base64 (
+                        const int accept_key_len = encode_base64 (
                           hash, SHA_DIGEST_LENGTH, _websocket_accept,
                           MAX_HEADER_VALUE_LENGTH);
                         assert (accept_key_len > 0);
                         _websocket_accept[accept_key_len] = '\0';
 
-                        int written =
-                          snprintf ((char *) _write_buffer, WS_BUFFER_SIZE,
+                        const int written =
+                          snprintf (reinterpret_cast<char *> (_write_buffer),
+                                    WS_BUFFER_SIZE,
                                     "HTTP/1.1 101 Switching Protocols\r\n"
                                     "Upgrade: websocket\r\n"
                                     "Connection: Upgrade\r\n"
@@ -443,8 +553,8 @@ bool zmq::ws_engine_t::server_handshake ()
                         _insize--;
 
                         return true;
-                    } else
-                        _server_handshake_state = handshake_error;
+                    }
+                    _server_handshake_state = handshake_error;
                 } else
                     _server_handshake_state = handshake_error;
                 break;
@@ -470,12 +580,8 @@ bool zmq::ws_engine_t::server_handshake ()
 
 bool zmq::ws_engine_t::client_handshake ()
 {
-    int nbytes = tcp_read (_read_buffer, WS_BUFFER_SIZE);
-    if (nbytes == 0) {
-        errno = EPIPE;
-        error (zmq::i_engine::connection_error);
-        return false;
-    } else if (nbytes == -1) {
+    const int nbytes = read (_read_buffer, WS_BUFFER_SIZE);
+    if (nbytes == -1) {
         if (errno != EAGAIN)
             error (zmq::i_engine::connection_error);
         return false;
@@ -485,7 +591,7 @@ bool zmq::ws_engine_t::client_handshake ()
     _insize = nbytes;
 
     while (_insize > 0) {
-        char c = (char) *_inpos;
+        const char c = static_cast<char> (*_inpos);
 
         switch (_client_handshake_state) {
             case client_handshake_initial:
@@ -707,7 +813,7 @@ bool zmq::ws_engine_t::client_handshake ()
                         _client_handshake_state = client_handshake_error;
                         break;
                     default:
-                        _header_name[0] = (char) c;
+                        _header_name[0] = c;
                         _header_name_position = 1;
                         _client_handshake_state = client_header_field_name;
                         break;
@@ -756,11 +862,11 @@ bool zmq::ws_engine_t::client_handshake ()
                           strcasecmp ("upgrade", _header_value) == 0;
                     else if (strcasecmp ("Sec-WebSocket-Accept", _header_name)
                              == 0)
-                        strcpy (_websocket_accept, _header_value);
+                        strcpy_s (_websocket_accept, _header_value);
                     else if (strcasecmp ("Sec-WebSocket-Protocol", _header_name)
                              == 0) {
-                        if (strcmp ("ZWS2.0", _header_value) == 0)
-                            strcpy (_websocket_protocol, _header_value);
+                        if (select_protocol (_header_value))
+                            strcpy_s (_websocket_protocol, _header_value);
                     }
                     _client_handshake_state = client_header_field_cr;
                 } else if (_header_value_position + 1 > MAX_HEADER_VALUE_LENGTH)
@@ -790,8 +896,8 @@ bool zmq::ws_engine_t::client_handshake ()
                         _insize--;
 
                         return true;
-                    } else
-                        _client_handshake_state = client_handshake_error;
+                    }
+                    _client_handshake_state = client_handshake_error;
                 } else
                     _client_handshake_state = client_handshake_error;
                 break;
@@ -814,46 +920,147 @@ bool zmq::ws_engine_t::client_handshake ()
     return false;
 }
 
+int zmq::ws_engine_t::decode_and_push (msg_t *msg_)
+{
+    zmq_assert (_mechanism != NULL);
+
+    //  with WS engine, ping and pong commands are control messages and should not go through any mechanism
+    if (msg_->is_ping () || msg_->is_pong () || msg_->is_close_cmd ()) {
+        if (process_command_message (msg_) == -1)
+            return -1;
+    } else if (_mechanism->decode (msg_) == -1)
+        return -1;
+
+    if (_has_timeout_timer) {
+        _has_timeout_timer = false;
+        cancel_timer (heartbeat_timeout_timer_id);
+    }
+
+    if (msg_->flags () & msg_t::command && !msg_->is_ping ()
+        && !msg_->is_pong () && !msg_->is_close_cmd ())
+        process_command_message (msg_);
+
+    if (_metadata)
+        msg_->set_metadata (_metadata);
+    if (session ()->push_msg (msg_) == -1) {
+        if (errno == EAGAIN)
+            _process_msg = &ws_engine_t::push_one_then_decode_and_push;
+        return -1;
+    }
+    return 0;
+}
+
+int zmq::ws_engine_t::produce_close_message (msg_t *msg_)
+{
+    int rc = msg_->move (_close_msg);
+    errno_assert (rc == 0);
+
+    _next_msg = static_cast<int (stream_engine_base_t::*) (msg_t *)> (
+      &ws_engine_t::produce_no_msg_after_close);
+
+    return rc;
+}
+
+int zmq::ws_engine_t::produce_no_msg_after_close (msg_t *msg_)
+{
+    _next_msg = static_cast<int (stream_engine_base_t::*) (msg_t *)> (
+      &ws_engine_t::close_connection_after_close);
+
+    errno = EAGAIN;
+    return -1;
+}
+
+int zmq::ws_engine_t::close_connection_after_close (msg_t *msg_)
+{
+    error (connection_error);
+    errno = ECONNRESET;
+    return -1;
+}
+
+int zmq::ws_engine_t::produce_ping_message (msg_t *msg_)
+{
+    int rc = msg_->init ();
+    errno_assert (rc == 0);
+    msg_->set_flags (msg_t::command | msg_t::ping);
+
+    _next_msg = &ws_engine_t::pull_and_encode;
+    if (!_has_timeout_timer && _heartbeat_timeout > 0) {
+        add_timer (_heartbeat_timeout, heartbeat_timeout_timer_id);
+        _has_timeout_timer = true;
+    }
+
+    return rc;
+}
+
+
+int zmq::ws_engine_t::produce_pong_message (msg_t *msg_)
+{
+    int rc = msg_->init ();
+    errno_assert (rc == 0);
+    msg_->set_flags (msg_t::command | msg_t::pong);
+
+    _next_msg = &ws_engine_t::pull_and_encode;
+    return rc;
+}
+
+
+int zmq::ws_engine_t::process_command_message (msg_t *msg_)
+{
+    if (msg_->is_ping ()) {
+        _next_msg = static_cast<int (stream_engine_base_t::*) (msg_t *)> (
+          &ws_engine_t::produce_pong_message);
+        out_event ();
+    } else if (msg_->is_close_cmd ()) {
+        int rc = _close_msg.copy (*msg_);
+        errno_assert (rc == 0);
+        _next_msg = static_cast<int (stream_engine_base_t::*) (msg_t *)> (
+          &ws_engine_t::produce_close_message);
+        out_event ();
+    }
+
+    return 0;
+}
+
 static int
-encode_base64 (const unsigned char *in, int in_len, char *out, int out_len)
+encode_base64 (const unsigned char *in_, int in_len_, char *out_, int out_len_)
 {
     static const unsigned char base64enc_tab[65] =
       "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 
-    int ii, io;
-    uint32_t v;
-    int rem;
+    int io = 0;
+    uint32_t v = 0;
+    int rem = 0;
 
-    for (io = 0, ii = 0, v = 0, rem = 0; ii < in_len; ii++) {
+    for (int ii = 0; ii < in_len_; ii++) {
         unsigned char ch;
-        ch = in[ii];
+        ch = in_[ii];
         v = (v << 8) | ch;
         rem += 8;
         while (rem >= 6) {
             rem -= 6;
-            if (io >= out_len)
+            if (io >= out_len_)
                 return -1; /* truncation is failure */
-            out[io++] = base64enc_tab[(v >> rem) & 63];
+            out_[io++] = base64enc_tab[(v >> rem) & 63];
         }
     }
     if (rem) {
         v <<= (6 - rem);
-        if (io >= out_len)
+        if (io >= out_len_)
             return -1; /* truncation is failure */
-        out[io++] = base64enc_tab[v & 63];
+        out_[io++] = base64enc_tab[v & 63];
     }
     while (io & 3) {
-        if (io >= out_len)
+        if (io >= out_len_)
             return -1; /* truncation is failure */
-        out[io++] = '=';
+        out_[io++] = '=';
     }
-    if (io >= out_len)
+    if (io >= out_len_)
         return -1; /* no room for null terminator */
-    out[io] = 0;
+    out_[io] = 0;
     return io;
 }
 
-static void compute_accept_key (char *key, unsigned char *hash)
+static void compute_accept_key (char *key_, unsigned char *hash_)
 {
     const char *magic_string = "258EAFA5-E914-47DA-95CA-C5AB0DC85B11";
 #ifdef ZMQ_USE_NSS
@@ -863,17 +1070,25 @@ static void compute_accept_key (char *key, unsigned char *hash)
     assert (ctx);
 
     HASH_Begin (ctx);
-    HASH_Update (ctx, (unsigned char *) key, (unsigned int) strlen (key));
+    HASH_Update (ctx, (unsigned char *) key_, (unsigned int) strlen (key_));
     HASH_Update (ctx, (unsigned char *) magic_string,
                  (unsigned int) strlen (magic_string));
-    HASH_End (ctx, hash, &len, SHA_DIGEST_LENGTH);
+    HASH_End (ctx, hash_, &len, SHA_DIGEST_LENGTH);
     HASH_Destroy (ctx);
-#else
+#elif defined ZMQ_USE_BUILTIN_SHA1
     sha1_ctxt ctx;
     SHA1_Init (&ctx);
-    SHA1_Update (&ctx, (unsigned char *) key, strlen (key));
+    SHA1_Update (&ctx, (unsigned char *) key_, strlen (key_));
     SHA1_Update (&ctx, (unsigned char *) magic_string, strlen (magic_string));
 
-    SHA1_Final (hash, &ctx);
+    SHA1_Final (hash_, &ctx);
+#elif defined ZMQ_USE_GNUTLS
+    gnutls_hash_hd_t hd;
+    gnutls_hash_init (&hd, GNUTLS_DIG_SHA1);
+    gnutls_hash (hd, key_, strlen (key_));
+    gnutls_hash (hd, magic_string, strlen (magic_string));
+    gnutls_hash_deinit (hd, hash_);
+#else
+#error "No sha1 implementation set"
 #endif
 }
